@@ -16,6 +16,7 @@
 #include <gtk/gtk.h>
 
 #include "process-statusbar.h"
+#include "settings.h"
 
 
 
@@ -24,6 +25,7 @@ enum
 	PROP_CPU = 1,
 	PROP_MEMORY,
 	PROP_SWAP,
+	PROP_SHOW_SWAP,
 	PROP_NUM_PROCESSES,
 };
 typedef struct _XtmProcessStatusbarClass XtmProcessStatusbarClass;
@@ -35,6 +37,8 @@ struct _XtmProcessStatusbar
 {
 	GtkStatusbar		parent;
 	/*<private>*/
+	XtmSettings *		settings;
+
 	GtkWidget *		label_num_processes;
 	GtkWidget *		label_cpu;
 	GtkWidget *		label_memory;
@@ -63,6 +67,8 @@ xtm_process_statusbar_class_init (XtmProcessStatusbarClass *klass)
 		g_param_spec_float ("memory", "Memory", "Memory usage", 0, 100, 0, G_PARAM_CONSTRUCT|G_PARAM_WRITABLE));
 	g_object_class_install_property (class, PROP_SWAP,
 		g_param_spec_float ("swap", "Swap", "Swap usage", 0, 100, 0, G_PARAM_CONSTRUCT|G_PARAM_WRITABLE));
+	g_object_class_install_property (class, PROP_SHOW_SWAP,
+		g_param_spec_boolean ("show-swap", "ShowSwap", "Show or hide swap usage", TRUE, G_PARAM_WRITABLE));
 	g_object_class_install_property (class, PROP_NUM_PROCESSES,
 		g_param_spec_uint ("num-processes", "NumProcesses", "Number of processes", 0, G_MAXUINT, 0, G_PARAM_CONSTRUCT|G_PARAM_WRITABLE));
 }
@@ -72,6 +78,8 @@ xtm_process_statusbar_init (XtmProcessStatusbar *statusbar)
 {
 	GtkWidget *area, *hbox;
 
+	statusbar->settings = xtm_settings_get_default ();
+
 #if GTK_CHECK_VERSION(2,20,0)
 	area = gtk_statusbar_get_message_area (GTK_STATUSBAR (statusbar));
 #else
@@ -91,7 +99,7 @@ xtm_process_statusbar_init (XtmProcessStatusbar *statusbar)
 #endif
 
 	hbox = gtk_hbox_new (FALSE, 16);
-	gtk_box_pack_start (GTK_BOX (area), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (area), hbox, TRUE, TRUE, 6);
 
 	statusbar->label_num_processes = gtk_label_new (NULL);
 	gtk_box_pack_start (GTK_BOX (hbox), statusbar->label_num_processes, FALSE, FALSE, 0);
@@ -108,35 +116,57 @@ xtm_process_statusbar_init (XtmProcessStatusbar *statusbar)
 	gtk_widget_show_all (hbox);
 }
 
+static gchar *
+rounded_float_value (gfloat value, XtmSettings *settings)
+{
+	gboolean precision;
+	g_object_get (settings, "more-precision", &precision, NULL);
+	return g_strdup_printf ((precision) ? "%.2f" : "%.0f", value);
+}
+
 static void
 xtm_process_statusbar_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
 {
 	XtmProcessStatusbar *statusbar = XTM_PROCESS_STATUSBAR (object);
 	gchar *text;
+	gchar *float_value;
 
 	switch (property_id)
 	{
 		case PROP_CPU:
 		statusbar->cpu = g_value_get_float (value);
-		text = g_strdup_printf (_("CPU: %.2f%%"), statusbar->cpu);
+		float_value = rounded_float_value (statusbar->cpu, statusbar->settings);
+		text = g_strdup_printf (_("CPU: %s%%"), float_value);
 		gtk_label_set_text (GTK_LABEL (statusbar->label_cpu), text);
+		g_free (float_value);
 		g_free (text);
 		break;
 
 		case PROP_MEMORY:
 		statusbar->memory = g_value_get_float (value);
-		text = g_strdup_printf (_("Memory: %.2f%%"), statusbar->memory);
+		float_value = rounded_float_value (statusbar->memory, statusbar->settings);
+		text = g_strdup_printf (_("Memory: %s%%"), float_value);
 		gtk_label_set_text (GTK_LABEL (statusbar->label_memory), text);
+		g_free (float_value);
 		g_free (text);
 		break;
 
 		case PROP_SWAP:
 		statusbar->swap = g_value_get_float (value);
-		text = g_strdup_printf (_("Swap: %.2f%%"), statusbar->swap);
+		float_value = rounded_float_value (statusbar->swap, statusbar->settings);
+		text = g_strdup_printf (_("Swap: %s%%"), float_value);
 		gtk_label_set_text (GTK_LABEL (statusbar->label_swap), text);
+		g_free (float_value);
 		g_free (text);
 		break;
 
+		case PROP_SHOW_SWAP:
+		if (g_value_get_boolean (value))
+			gtk_widget_show (statusbar->label_swap);
+		else
+			gtk_widget_hide (statusbar->label_swap);
+		break;
+
 		case PROP_NUM_PROCESSES:
 		statusbar->num_processes = g_value_get_uint (value);
 		text = g_strdup_printf (_("Processes: %d"), statusbar->num_processes);
