@@ -96,6 +96,15 @@ xtm_process_window_init (XtmProcessWindow *window)
 	window->priv->cpu_monitor = GTK_WIDGET (gtk_builder_get_object (window->priv->builder, "cpu-monitor"));
 	window->priv->memory_monitor = GTK_WIDGET (gtk_builder_get_object (window->priv->builder, "mem-monitor"));
 
+	if (geteuid () == 0)
+	{
+		gtk_rc_parse_string ("style\"root-warning-style\"{bg[NORMAL]=\"#b4254b\"\nfg[NORMAL]=\"#fefefe\"}\n"
+				"widget\"GtkWindow.*.root-warning\"style\"root-warning-style\"\n"
+				"widget\"GtkWindow.*.root-warning.GtkLabel\"style\"root-warning-style\"");
+		gtk_widget_set_name (GTK_WIDGET (gtk_builder_get_object (window->priv->builder, "root-warning-ebox")), "root-warning");
+		gtk_widget_show_all (GTK_WIDGET (gtk_builder_get_object (window->priv->builder, "root-warning-box")));
+	}
+
 	window->priv->treeview = xtm_process_tree_view_new ();
 	gtk_widget_show (window->priv->treeview);
 	gtk_container_add (GTK_CONTAINER (gtk_builder_get_object (window->priv->builder, "scrolledwindow")), window->priv->treeview);
@@ -182,25 +191,90 @@ menu_execute_append_item (GtkMenu *menu, gchar *title, gchar *command, gchar *ic
 	g_signal_connect_swapped (mi, "activate", G_CALLBACK (execute_command), command);
 }
 
+static gboolean
+program_exists (gchar *program)
+{
+	gchar *program_path = g_find_program_in_path (program);
+	if (program_path == NULL)
+		return FALSE;
+	g_free (program_path);
+	return TRUE;
+}
+
 static void
 show_menu_execute_task (XtmProcessWindow *window)
 {
-	// TODO check if xfrun4, xfce4-appfinder, etc are installed and pull them in the menu
 	static GtkWidget *menu = NULL;
 
 	if (menu == NULL)
 	{
 		menu = gtk_menu_new ();
-		menu_execute_append_item (GTK_MENU (menu), _("Run Program..."), "xfrun4", "system-run");
-		menu_execute_append_item (GTK_MENU (menu), _("Application Finder"), "xfce4-appfinder", "xfce4-appfinder");
-		menu_execute_append_item (GTK_MENU (menu), _("Terminal emulator"), "exo-open --launch TerminalEmulator", "terminal");
-		menu_execute_append_item (GTK_MENU (menu), _("XTerm"), "xterm -fg grey -bg black", "terminal");
+		/* Find a runner program */
+		if (program_exists ("xfrun4"))
+			menu_execute_append_item (GTK_MENU (menu), _("Run Program..."), "xfrun4", "system-run");
+		else if (program_exists ("gmrun"))
+			menu_execute_append_item (GTK_MENU (menu), _("Run Program..."), "gmrun", "system-run");
+		else if (program_exists ("gexec"))
+			menu_execute_append_item (GTK_MENU (menu), _("Run Program..."), "gexec", "system-run");
+		/* Find an applications list program */
+		if (program_exists ("xfce4-appfinder"))
+			menu_execute_append_item (GTK_MENU (menu), _("Application Finder"), "xfce4-appfinder", "xfce4-appfinder");
+		/* Find a terminal emulator */
+		if (program_exists ("exo-open"))
+			menu_execute_append_item (GTK_MENU (menu), _("Terminal emulator"), "exo-open --launch TerminalEmulator", "terminal");
+		else if (program_exists ("xterm"))
+			menu_execute_append_item (GTK_MENU (menu), _("XTerm"), "xterm -fg grey -bg black", "terminal");
 		gtk_widget_show_all (menu);
 	}
 
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time ());
 }
 
+static void
+refresh_rate_toggled (GtkCheckMenuItem *mi, XtmSettings *settings)
+{
+	guint refresh_rate = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (mi), "refresh-rate"));
+	g_object_set (settings, "refresh-rate", refresh_rate, NULL);
+}
+
+static void
+menu_refresh_rate_append_item (GtkMenu *menu, gchar *title, guint refresh_rate, XtmSettings *settings)
+{
+	GtkWidget *mi;
+	guint cur_refresh_rate;
+
+	g_object_get (settings, "refresh-rate", &cur_refresh_rate, NULL);
+
+	mi = gtk_image_menu_item_new_with_label (title);
+	if (cur_refresh_rate == refresh_rate)
+	{
+		GtkWidget *image = gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_MENU);
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+	}
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+	g_object_set_data (G_OBJECT (mi), "refresh-rate", GUINT_TO_POINTER (refresh_rate));
+	g_signal_connect (mi, "activate", G_CALLBACK (refresh_rate_toggled), settings);
+}
+
+static GtkWidget *
+build_refresh_rate_menu (XtmSettings *settings)
+{
+	GtkWidget *menu;
+
+	menu = gtk_menu_new ();
+
+	/* TRANSLATORS: The next values are in seconds or milliseconds */
+	menu_refresh_rate_append_item (GTK_MENU (menu), _("250ms"), 250, settings);
+	menu_refresh_rate_append_item (GTK_MENU (menu), _("500ms"), 500, settings);
+	menu_refresh_rate_append_item (GTK_MENU (menu), _("750ms"), 750, settings);
+	menu_refresh_rate_append_item (GTK_MENU (menu), _("1s"), 1000, settings);
+	menu_refresh_rate_append_item (GTK_MENU (menu), _("2s"), 2000, settings);
+	menu_refresh_rate_append_item (GTK_MENU (menu), _("5s"), 5000, settings);
+	menu_refresh_rate_append_item (GTK_MENU (menu), _("10s"), 10000, settings);
+
+	return menu;
+}
+
 static void
 preferences_toggled (GtkCheckMenuItem *mi, XtmSettings *settings)
 {
@@ -228,6 +302,7 @@ static void
 show_menu_preferences (XtmProcessWindow *window)
 {
 	static GtkWidget *menu = NULL;
+	GtkWidget *refresh_rate_menu;
 	GtkWidget *mi;
 
 	if (menu != NULL)
@@ -237,6 +312,14 @@ show_menu_preferences (XtmProcessWindow *window)
 
 	menu = gtk_menu_new ();
 	menu_preferences_append_item (GTK_MENU (menu), _("Show all processes"), "show-all-processes", window->priv->settings);
+	menu_preferences_append_item (GTK_MENU (menu), _("More precision"), "more-precision", window->priv->settings);
+	menu_preferences_append_item (GTK_MENU (menu), _("Full command line"), "full-command-line", window->priv->settings);
+	menu_preferences_append_item (GTK_MENU (menu), _("Show status icon"), "show-status-icon", window->priv->settings);
+
+	refresh_rate_menu = build_refresh_rate_menu (window->priv->settings);
+	mi = gtk_menu_item_new_with_label (_("Refresh rate"));
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), refresh_rate_menu);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
 
 	mi = gtk_separator_menu_item_new ();
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
@@ -333,14 +416,20 @@ xtm_process_window_show (GtkWidget *widget)
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 	g_return_if_fail (GTK_IS_WIDGET (XTM_PROCESS_WINDOW (widget)->priv->window));
 	gtk_widget_show (XTM_PROCESS_WINDOW (widget)->priv->window);
+	gtk_window_present (GTK_WINDOW (XTM_PROCESS_WINDOW (widget)->priv->window));
+	GTK_WIDGET_SET_FLAGS (widget, GTK_VISIBLE);
 }
 
 static void
 xtm_process_window_hide (GtkWidget *widget)
 {
+	gint winx, winy;
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 	g_return_if_fail (GTK_IS_WIDGET (XTM_PROCESS_WINDOW (widget)->priv->window));
+	gtk_window_get_position (GTK_WINDOW (XTM_PROCESS_WINDOW (widget)->priv->window), &winx, &winy);
 	gtk_widget_hide (XTM_PROCESS_WINDOW (widget)->priv->window);
+	gtk_window_move (GTK_WINDOW (XTM_PROCESS_WINDOW (widget)->priv->window), winx, winy);
+	GTK_WIDGET_UNSET_FLAGS (widget, GTK_VISIBLE);
 }
 
 GtkTreeModel *
@@ -362,3 +451,11 @@ xtm_process_window_set_system_info (XtmProcessWindow *window, guint num_processe
 	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (window->priv->cpu_monitor), cpu / 100.0);
 }
 
+void
+xtm_process_window_show_swap_usage (XtmProcessWindow *window, gboolean show_swap_usage)
+{
+	g_return_if_fail (XTM_IS_PROCESS_WINDOW (window));
+	g_return_if_fail (GTK_IS_STATUSBAR (window->priv->statusbar));
+	g_object_set (window->priv->statusbar, "show-swap", show_swap_usage, NULL);
+}
+
