@@ -23,6 +23,16 @@
 
 #include "task-manager.h"
 #include "process-tree-view.h" /* for the columns of the model */
+#include "settings.h"
+
+#define TIMESTAMP_DELTA 4
+
+
+
+static XtmSettings *settings = NULL;
+static gboolean model_update_forced = FALSE;
+static gboolean more_precision;
+static gboolean full_cmdline;
 
 
 
@@ -54,10 +64,14 @@ G_DEFINE_TYPE (XtmTaskManager, xtm_task_manager, G_TYPE_OBJECT)
 
 static void	xtm_task_manager_finalize			(GObject *object);
 
-static void	model_add_task					(GtkTreeModel *model, Task *task);
+static void	setting_changed					(GObject *object, GParamSpec *pspec, XtmTaskManager *manager);
+static void	model_add_task					(GtkTreeModel *model, Task *task, glong timestamp);
+static void	model_mark_tree_iter_as_removed			(GtkTreeModel *model, GtkTreeIter *iter);
+static void	model_remove_tree_iter				(GtkTreeModel *model, GtkTreeIter *iter);
 static void	model_update_tree_iter				(GtkTreeModel *model, GtkTreeIter *iter, Task *task);
 static void	model_update_task				(GtkTreeModel *model, Task *task);
-static void	model_remove_task				(GtkTreeModel *model, Task *task);
+static void	model_find_tree_iter_for_pid			(GtkTreeModel *model, guint pid, GtkTreeIter *iter);
+static glong	__current_timestamp				();
 
 
 
@@ -75,6 +89,13 @@ xtm_task_manager_init (XtmTaskManager *manager)
 	manager->tasks = g_array_new (FALSE, FALSE, sizeof (Task));
 	get_owner_uid (&(manager->owner_uid), &(manager->owner_uid_name));
 	manager->hostname = get_hostname ();
+
+	/* Listen to settings changes and force an update on the whole model */
+	settings = xtm_settings_get_default ();
+	g_object_get (settings, "more-precision", &more_precision, NULL);
+	g_object_get (settings, "full-command-line", &full_cmdline, NULL);
+	g_signal_connect (settings, "notify::more-precision", G_CALLBACK (setting_changed), manager);
+	g_signal_connect (settings, "notify::full-command-line", G_CALLBACK (setting_changed), manager);
 }
 
 static void
@@ -86,6 +107,33 @@ xtm_task_manager_finalize (GObject *object)
 	g_free (manager->hostname);
 }
 
+static void
+setting_changed (GObject *object, GParamSpec *pspec, XtmTaskManager *manager)
+{
+	g_object_get (object, "more-precision", &more_precision, NULL);
+	g_object_get (object, "full-command-line", &full_cmdline, NULL);
+	model_update_forced = TRUE;
+}
+
+static gchar *
+pretty_cmdline (gchar *cmdline, gchar *comm)
+{
+	gchar *text = g_strchomp (g_strdelimit (g_strdup (cmdline), "\n\r", ' '));
+	if (!full_cmdline && sizeof (text) > 3)
+	{
+		/* Shorten full path to commands and wine applications */
+		if (text[0] == '/' || (g_ascii_isupper (text[0]) && text[1] == ':' && text[2] == '\\'))
+		{
+			gchar *p = g_strstr_len (text, -1, comm);
+			if (p != NULL)
+			{
+				g_snprintf (text, g_utf8_strlen (text, -1), p);
+			}
+		}
+	}
+	return text;
+}
+
 static void
 _xtm_task_manager_set_model (XtmTaskManager *manager, GtkTreeModel *model)
 {
@@ -95,18 +143,41 @@ _xtm_task_manager_set_model (XtmTaskManager *manager, GtkTreeModel *model)
 }
 
 static void
-model_add_task (GtkTreeModel *model, Task *task)
+model_add_task (GtkTreeModel *model, Task *task, glong timestamp)
 {
 	GtkTreeIter iter;
+	gchar *cmdline = pretty_cmdline (task->cmdline, task->name);
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-		XTM_PTV_COLUMN_COMMAND, task->cmdline,
+		XTM_PTV_COLUMN_COMMAND, cmdline,
 		XTM_PTV_COLUMN_PID, task->pid,
 		XTM_PTV_COLUMN_STATE, task->state,
 		XTM_PTV_COLUMN_UID, task->uid,
 		XTM_PTV_COLUMN_UID_STR, task->uid_name,
+		XTM_PTV_COLUMN_BACKGROUND, NULL,
+		XTM_PTV_COLUMN_FOREGROUND, NULL,
+		XTM_PTV_COLUMN_TIMESTAMP, timestamp,
 		-1);
 	model_update_tree_iter (model, &iter, task);
+	g_free (cmdline);
+}
+
+static void
+model_mark_tree_iter_as_removed (GtkTreeModel *model, GtkTreeIter *iter)
+{
+	gtk_list_store_set (GTK_LIST_STORE (model), iter,
+		XTM_PTV_COLUMN_CPU, 0.0,
+		XTM_PTV_COLUMN_CPU_STR, "-",
+		XTM_PTV_COLUMN_BACKGROUND, "#a40000",
+		XTM_PTV_COLUMN_FOREGROUND, "#ffffff",
+		XTM_PTV_COLUMN_TIMESTAMP, __current_timestamp (),
+		-1);
+}
+
+static void
+model_remove_tree_iter (GtkTreeModel *model, GtkTreeIter *iter)
+{
+	gtk_list_store_remove (GTK_LIST_STORE (model), iter);
 }
 
 static void
@@ -135,11 +206,48 @@ static void
 model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 {
 	gchar vsz[64], rss[64], cpu[16];
+	gchar value[14];
+	glong old_timestamp;
+	gchar *old_state;
+	gchar *background, *foreground;
 
 	memory_human_size (task->vsz, vsz);
 	memory_human_size (task->rss, rss);
-	// TODO make precision optional
-	g_snprintf (cpu, 16, _("%.2f%%"), task->cpu_user + task->cpu_system);
+
+	g_snprintf (value, 14, (more_precision) ? "%.2f" : "%.0f", task->cpu_user + task->cpu_system);
+	g_snprintf (cpu, 16, _("%s%%"), value);
+
+	if (model_update_forced)
+	{
+		gchar *cmdline = pretty_cmdline (task->cmdline, task->name);
+		gtk_list_store_set (GTK_LIST_STORE (model), iter, XTM_PTV_COLUMN_COMMAND, cmdline, -1);
+		g_free (cmdline);
+	}
+
+	/* Retrieve values needed for tweaking background/foreground color */
+	gtk_tree_model_get (model, iter, XTM_PTV_COLUMN_TIMESTAMP, &old_timestamp, XTM_PTV_COLUMN_STATE, &old_state,
+			XTM_PTV_COLUMN_BACKGROUND, &background, XTM_PTV_COLUMN_FOREGROUND, &foreground, -1);
+
+	if (g_strcmp0 (task->state, old_state) != 0 && background == NULL)
+	{
+		/* Set yellow color for changing state */
+		background = g_strdup ("#edd400");
+		foreground = g_strdup ("#000000");
+		old_timestamp = __current_timestamp () - TIMESTAMP_DELTA + 3;
+	}
+
+	if (__current_timestamp () - old_timestamp <= TIMESTAMP_DELTA)
+	{
+		/* Set green color for started task */
+		background = (background == NULL) ? g_strdup ("#73d216") : background;
+		foreground = (foreground == NULL) ? g_strdup ("#000000") : foreground;
+	}
+	else
+	{
+		/* Reset color */
+		background = NULL;
+		foreground = NULL;
+	}
 
 	gtk_list_store_set (GTK_LIST_STORE (model), iter,
 		XTM_PTV_COLUMN_PPID, task->ppid,
@@ -151,7 +259,21 @@ model_update_tree_iter (GtkTreeModel *model, GtkTreeIter *iter, Task *task)
 		XTM_PTV_COLUMN_CPU, task->cpu_user + task->cpu_system,
 		XTM_PTV_COLUMN_CPU_STR, cpu,
 		XTM_PTV_COLUMN_PRIORITY, task->prio,
+		XTM_PTV_COLUMN_BACKGROUND, background,
+		XTM_PTV_COLUMN_FOREGROUND, foreground,
+		XTM_PTV_COLUMN_TIMESTAMP, old_timestamp,
 		-1);
+
+	g_free (background);
+	g_free (foreground);
+}
+
+static void
+model_update_task (GtkTreeModel *model, Task *task)
+{
+	GtkTreeIter iter;
+	model_find_tree_iter_for_pid (model, task->pid, &iter);
+	model_update_tree_iter (model, &iter, task);
 }
 
 static void
@@ -170,20 +292,12 @@ model_find_tree_iter_for_pid (GtkTreeModel *model, guint pid, GtkTreeIter *iter)
 	}
 }
 
-static void
-model_update_task (GtkTreeModel *model, Task *task)
+static glong
+__current_timestamp ()
 {
-	GtkTreeIter iter;
-	model_find_tree_iter_for_pid (model, task->pid, &iter);
-	model_update_tree_iter (model, &iter, task);
-}
-
-static void
-model_remove_task (GtkTreeModel *model, Task *task)
-{
-	GtkTreeIter iter;
-	model_find_tree_iter_for_pid (model, task->pid, &iter);
-	gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+	GTimeVal time;
+	g_get_current_time (&time);
+	return time.tv_sec;
 }
 
 
@@ -215,6 +329,8 @@ xtm_task_manager_get_system_info (XtmTaskManager *manager, guint *num_processes,
 {
 	guint64 memory_used, swap_used;
 
+	g_return_if_fail (XTM_IS_TASK_MANAGER (manager));
+
 	/* Set number of processes */
 	*num_processes = manager->tasks->len;
 
@@ -233,9 +349,18 @@ xtm_task_manager_get_system_info (XtmTaskManager *manager, guint *num_processes,
 	*cpu = manager->cpu_user + manager->cpu_system;
 }
 
+void
+xtm_task_manager_get_swap_usage (XtmTaskManager *manager, guint64 *swap_free, guint64 *swap_total)
+{
+	g_return_if_fail (XTM_IS_TASK_MANAGER (manager));
+	*swap_free = manager->swap_free;
+	*swap_total = manager->swap_total;
+}
+
 const GArray *
 xtm_task_manager_get_task_list (XtmTaskManager *manager)
 {
+	g_return_val_if_fail (XTM_IS_TASK_MANAGER (manager), NULL);
 	xtm_task_manager_update_model (manager);
 	return manager->tasks;
 }
@@ -243,9 +368,15 @@ xtm_task_manager_get_task_list (XtmTaskManager *manager)
 void
 xtm_task_manager_update_model (XtmTaskManager *manager)
 {
+	static GArray *removed_tasks = NULL;
 	GArray *array;
 	guint i;
 
+	g_return_if_fail (XTM_IS_TASK_MANAGER (manager));
+
+	if (removed_tasks == NULL)
+		removed_tasks = g_array_new (FALSE, FALSE, sizeof (GtkTreeIter));
+
 	/* Retrieve initial task list and return */
 	if (manager->tasks->len == 0)
 	{
@@ -253,8 +384,8 @@ xtm_task_manager_update_model (XtmTaskManager *manager)
 		for (i = 0; i < manager->tasks->len; i++)
 		{
 			Task *task = &g_array_index (manager->tasks, Task, i);
-			model_add_task (manager->model, task);
-#if 1|DEBUG
+			model_add_task (manager->model, task, 0);
+#if DEBUG
 			g_print ("%5d %5s %15s %.50s\n", task->pid, task->uid_name, task->name, task->cmdline);
 #endif
 		}
@@ -266,6 +397,23 @@ xtm_task_manager_update_model (XtmTaskManager *manager)
 	get_task_list (array);
 
 	/* Remove terminated tasks */
+	for (i = 0; i < removed_tasks->len; i++)
+	{
+		GtkTreeIter *iter = &g_array_index (removed_tasks, GtkTreeIter, i);
+		glong old_timestamp;
+		gtk_tree_model_get (manager->model, iter, XTM_PTV_COLUMN_TIMESTAMP, &old_timestamp, -1);
+		if (__current_timestamp () - old_timestamp > TIMESTAMP_DELTA)
+		{
+#if DEBUG
+			gint pid;
+			gtk_tree_model_get (manager->model, iter, XTM_PTV_COLUMN_PID, &pid, -1);
+			g_debug ("Remove old task %d", pid);
+#endif
+			model_remove_tree_iter (manager->model, iter);
+			g_array_remove_index (removed_tasks, i);
+		}
+	}
+
 	for (i = 0; i < manager->tasks->len; i++)
 	{
 		guint j;
@@ -284,10 +432,13 @@ xtm_task_manager_update_model (XtmTaskManager *manager)
 
 		if (found == FALSE)
 		{
+			GtkTreeIter iter;
 #if DEBUG
-			g_debug ("Remove old task %d %s", task->pid, task->name);
+			g_debug ("Task %d '%s' terminated, marking as removed", task->pid, task->name);
 #endif
-			model_remove_task (manager->model, task);
+			model_find_tree_iter_for_pid (manager->model, task->pid, &iter);
+			model_mark_tree_iter_as_removed (manager->model, &iter);
+			g_array_append_val (removed_tasks, iter);
 			g_array_remove_index (manager->tasks, i);
 		}
 	}
@@ -302,13 +453,16 @@ xtm_task_manager_update_model (XtmTaskManager *manager)
 		for (j = 0; j < manager->tasks->len; j++)
 		{
 			Task *task = &g_array_index (manager->tasks, Task, j);
+			gboolean updated = FALSE;
+
 			if (task->pid != tasktmp->pid)
 				continue;
 
 			found = TRUE;
 
 			/* Update the model (with the rest) only if needed, this keeps the CPU cool */
-			if (task->ppid != tasktmp->ppid
+			if (model_update_forced
+				|| task->ppid != tasktmp->ppid
 				|| g_strcmp0 (task->state, tasktmp->state)
 				|| task->cpu_user != tasktmp->cpu_user
 				|| task->cpu_system != tasktmp->cpu_system
@@ -316,6 +470,7 @@ xtm_task_manager_update_model (XtmTaskManager *manager)
 				|| task->vsz != tasktmp->vsz
 				|| task->prio != tasktmp->prio)
 			{
+				updated = TRUE;
 				task->ppid = tasktmp->ppid;
 				g_strlcpy (task->state, tasktmp->state, sizeof (task->state));
 				task->cpu_user = tasktmp->cpu_user;
@@ -326,6 +481,27 @@ xtm_task_manager_update_model (XtmTaskManager *manager)
 				model_update_task (manager->model, tasktmp);
 			}
 
+			/* Update color if needed */
+			if (updated == FALSE)
+			{
+				GtkTreeIter iter;
+				gchar *color;
+				glong old_timestamp;
+
+				model_find_tree_iter_for_pid (manager->model, task->pid, &iter);
+				gtk_tree_model_get (manager->model, &iter, XTM_PTV_COLUMN_BACKGROUND, &color, XTM_PTV_COLUMN_TIMESTAMP, &old_timestamp, -1);
+
+				if (color != NULL && __current_timestamp () - old_timestamp > TIMESTAMP_DELTA)
+				{
+#if DEBUG
+					g_debug ("Remove color from running PID %d", task->pid);
+#endif
+					model_update_task (manager->model, tasktmp);
+				}
+
+				g_free (color);
+			}
+
 			break;
 		}
 
@@ -334,12 +510,13 @@ xtm_task_manager_update_model (XtmTaskManager *manager)
 #if DEBUG
 			g_debug ("Add new task %d %s", tasktmp->pid, tasktmp->name);
 #endif
-			model_add_task (manager->model, tasktmp);
+			model_add_task (manager->model, tasktmp, __current_timestamp ());
 			g_array_append_val (manager->tasks, *tasktmp);
 		}
 	}
 
 	g_array_free (array, TRUE);
+	model_update_forced = FALSE;
 
 	return;
 }
