@@ -38,27 +38,6 @@ struct _Task
 	gshort		prio;
 };
 
-/**
- * Enumerations of virtual values between the interface and the OS implementation.
- */
-
-enum
-{
-	XTM_SIGNAL_TERMINATE = 0,
-	XTM_SIGNAL_STOP,
-	XTM_SIGNAL_CONTINUE,
-	XTM_SIGNAL_KILL,
-};
-
-enum
-{
-	XTM_PRIORITY_VERY_LOW = 0,
-	XTM_PRIORITY_LOW,
-	XTM_PRIORITY_NORMAL,
-	XTM_PRIORITY_HIGH,
-	XTM_PRIORITY_VERY_HIGH,
-};
-
 /**
  * OS specific implementation.
  */
@@ -86,6 +65,7 @@ XtmTaskManager *	xtm_task_manager_new				(GtkTreeModel *model);
 const gchar *		xtm_task_manager_get_username			(XtmTaskManager *manager);
 const gchar *		xtm_task_manager_get_hostname			(XtmTaskManager *manager);
 void			xtm_task_manager_get_system_info		(XtmTaskManager *manager, guint *num_processes, gfloat *cpu, gfloat *memory, gfloat *swap);
+void			xtm_task_manager_get_swap_usage			(XtmTaskManager *manager, guint64 *swap_free, guint64 *swap_total);
 const GArray *		xtm_task_manager_get_task_list			(XtmTaskManager *manager);
 void			xtm_task_manager_update_model			(XtmTaskManager *manager);
 
@@ -93,6 +73,23 @@ void			xtm_task_manager_update_model			(XtmTaskManager *manager);
  * Helper functions.
  */
 
+enum
+{
+	XTM_SIGNAL_TERMINATE = 0,
+	XTM_SIGNAL_STOP,
+	XTM_SIGNAL_CONTINUE,
+	XTM_SIGNAL_KILL,
+};
+
+enum
+{
+	XTM_PRIORITY_VERY_LOW = 0,
+	XTM_PRIORITY_LOW,
+	XTM_PRIORITY_NORMAL,
+	XTM_PRIORITY_HIGH,
+	XTM_PRIORITY_VERY_HIGH,
+};
+
 void		get_owner_uid		(guint *owner_uid, gchar **owner_uid_name);
 gchar *		get_hostname		();
 gboolean	send_signal_to_pid	(guint pid, gint signal);
